/**
 * Copyright (c) 2013-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @providesModule URI
 * @flow
 */

'use strict';

class URI {
  _uri: string;

  constructor(uri: string) {
    this._uri = uri;
  }

  toString(): string {
    return this._uri;
  }
}

module.exports = URI;
