// Copyright 2004-present Facebook. All Rights Reserved.

'use strict';

module.exports = {
  singleQuote: true,
  trailingComma: 'all',
  bracketSpacing: false,
  jsxBracketSameLine: true,
  parser: 'flow',
  requirePragma: true,
};
